@@ -451,7 +451,6 @@ export const findActiveDhcp = (name) => async (dispatch, getState) => {
             dispatch(addErrorToast({ error: 'dhcp_static_ip_error' }));
         }
 
-
         if (isError) {
             dispatch(addErrorToast({ error: 'dhcp_error' }));
         }
@@ -547,6 +546,22 @@ export const resetDhcp = () => async (dispatch) => {
     }
 };
 
+export const resetDhcpLeasesRequest = createAction('RESET_DHCP_LEASES_REQUEST');
+export const resetDhcpLeasesSuccess = createAction('RESET_DHCP_LEASES_SUCCESS');
+export const resetDhcpLeasesFailure = createAction('RESET_DHCP_LEASES_FAILURE');
+
+export const resetDhcpLeases = () => async (dispatch) => {
+    dispatch(resetDhcpLeasesRequest());
+    try {
+        const status = await apiClient.resetDhcpLeases();
+        dispatch(resetDhcpLeasesSuccess(status));
+        dispatch(addSuccessToast('dhcp_reset_leases_success'));
+    } catch (error) {
+        dispatch(addErrorToast({ error }));
+        dispatch(resetDhcpLeasesFailure());
+    }
+};
+
 export const toggleLeaseModal = createAction('TOGGLE_LEASE_MODAL');
 
 export const addStaticLeaseRequest = createAction('ADD_STATIC_LEASE_REQUEST');
@@ -561,6 +576,7 @@ export const addStaticLease = (config) => async (dispatch) => {
         dispatch(addStaticLeaseSuccess(config));
         dispatch(addSuccessToast(i18next.t('dhcp_lease_added', { key: name })));
         dispatch(toggleLeaseModal());
+        dispatch(getDhcpStatus());
     } catch (error) {
         dispatch(addErrorToast({ error }));
         dispatch(addStaticLeaseFailure());
@@ -621,7 +637,11 @@ export const toggleBlocking = (
 };
 
 export const toggleBlockingForClient = (type, domain, client) => {
-    const baseRule = `||${domain}^$client='${client.replace(/'/g, '/\'')}'`;
+    const escapedClientName = client.replace(/'/g, '\\\'')
+        .replace(/"/g, '\\"')
+        .replace(/,/g, '\\,')
+        .replace(/\|/g, '\\|');
+    const baseRule = `||${domain}^$client='${escapedClientName}'`;
     const baseUnblocking = `@@${baseRule}`;
 
     return toggleBlocking(type, domain, baseRule, baseUnblocking);
